"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseAnonCredsProof = exports.markProofAsViewed = exports.linkProofWithTemplate = exports.isPresentationReceived = exports.isPresentationFailed = exports.groupSharedProofDataByCredential = exports.getProofIdentifiers = exports.getProofData = void 0;
var _core = require("@credo-ts/core");
var _metadata = require("../types/metadata");
var _proof = require("../types/proof");
/*
 * Extract identifiers from indy proof
 * */
const getProofIdentifiers = (proof, proofIndex) => {
  const identifiers = proof.identifiers[proofIndex];
  if (!identifiers) {
    throw new Error('Invalid indy proof data');
  }
  return identifiers;
};

/*
 * Process anoncreds proof and return
 *  - shared attributes
 *  - shared attribute groups
 *  - resolved predicates
 *  - missing attributes
 *  - missing attribute groups
 *  - unresolved predicates
 * */
exports.getProofIdentifiers = getProofIdentifiers;
const parseAnonCredsProof = (request, proof) => {
  const result = new _proof.ParsedAnonCredsProof();
  for (const [referent, requested_attribute] of Object.entries(request.requested_attributes)) {
    if (requested_attribute.name) {
      const shared = proof.requested_proof.revealed_attrs[referent];
      if (shared) {
        const identifiers = getProofIdentifiers(proof, shared.sub_proof_index);
        result.sharedAttributes.push({
          name: requested_attribute.name,
          value: shared.raw,
          identifiers
        });
      } else {
        result.unresolvedAttributes.push({
          name: requested_attribute.name
        });
      }
    }
    if (requested_attribute.names) {
      var _proof$requested_proo;
      const shared = (_proof$requested_proo = proof.requested_proof.revealed_attr_groups) === null || _proof$requested_proo === void 0 ? void 0 : _proof$requested_proo[referent];
      if (shared) {
        const attributes = Object.entries(shared.values).map(_ref => {
          let [name, value] = _ref;
          return {
            name,
            value: value.raw
          };
        });
        const identifiers = getProofIdentifiers(proof, shared.sub_proof_index);
        result.sharedAttributeGroups.push({
          attributes,
          identifiers
        });
      } else {
        result.unresolvedAttributeGroups.push(requested_attribute.names.map(name => ({
          name
        })));
      }
    }
  }
  for (const [referent, requestedPredicate] of Object.entries(request.requested_predicates)) {
    // @ts-ignore Mistake in Credo type definition
    const shared = proof.requested_proof.predicates[referent];
    if (shared) {
      const identifiers = getProofIdentifiers(proof, shared.sub_proof_index);
      result.resolvedPredicates.push({
        name: requestedPredicate.name,
        predicateType: requestedPredicate.p_type,
        predicateValue: requestedPredicate.p_value,
        identifiers
      });
    } else {
      result.unresolvedPredicates.push({
        name: requestedPredicate.name,
        predicateType: requestedPredicate.p_type,
        predicateValue: requestedPredicate.p_value
      });
    }
  }
  return result;
};

/*
 * Group parsed indy proof data ( returned from `parseIndyProof`) by credential definition id
 * */
exports.parseAnonCredsProof = parseAnonCredsProof;
const groupSharedProofDataByCredential = data => {
  const result = new Map();
  for (const item of data.sharedAttributes) {
    var _result$get;
    if (!result.has(item.identifiers.cred_def_id)) {
      result.set(item.identifiers.cred_def_id, {
        data: new _proof.CredentialSharedProofData(),
        identifiers: item.identifiers
      });
    }
    (_result$get = result.get(item.identifiers.cred_def_id)) === null || _result$get === void 0 ? void 0 : _result$get.data.sharedAttributes.push(item);
  }
  for (const item of data.sharedAttributeGroups) {
    var _result$get2;
    if (!result.has(item.identifiers.cred_def_id)) {
      result.set(item.identifiers.cred_def_id, {
        data: new _proof.CredentialSharedProofData(),
        identifiers: item.identifiers
      });
    }
    (_result$get2 = result.get(item.identifiers.cred_def_id)) === null || _result$get2 === void 0 ? void 0 : _result$get2.data.sharedAttributeGroups.push(item);
  }
  for (const item of data.resolvedPredicates) {
    var _result$get3;
    if (!result.has(item.identifiers.cred_def_id)) {
      result.set(item.identifiers.cred_def_id, {
        data: new _proof.CredentialSharedProofData(),
        identifiers: item.identifiers
      });
    }
    (_result$get3 = result.get(item.identifiers.cred_def_id)) === null || _result$get3 === void 0 ? void 0 : _result$get3.data.resolvedPredicates.push(item);
  }
  return result;
};

/*
 * Retrieve proof details from Credo record
 * */
exports.groupSharedProofDataByCredential = groupSharedProofDataByCredential;
const getProofData = async (agent, recordId) => {
  var _data$request, _data$presentation, _data$request2, _data$presentation2;
  const data = await agent.proofs.getFormatData(recordId);
  if ((_data$request = data.request) !== null && _data$request !== void 0 && _data$request.anoncreds && (_data$presentation = data.presentation) !== null && _data$presentation !== void 0 && _data$presentation.anoncreds) {
    return parseAnonCredsProof(data.request.anoncreds, data.presentation.anoncreds);
  } else if ((_data$request2 = data.request) !== null && _data$request2 !== void 0 && _data$request2.indy && (_data$presentation2 = data.presentation) !== null && _data$presentation2 !== void 0 && _data$presentation2.indy) {
    return parseAnonCredsProof(data.request.indy, data.presentation.indy);
  }
  return undefined;
};

/*
 * Check if a presentation received
 * */
exports.getProofData = getProofData;
const isPresentationReceived = record => {
  return record.state === _core.ProofState.PresentationReceived || record.state === _core.ProofState.Done;
};

/*
 * Check if a presentation failed
 * */
exports.isPresentationReceived = isPresentationReceived;
const isPresentationFailed = record => {
  return record.state === _core.ProofState.Abandoned;
};

/*
 * Mark Proof record as viewed
 * */
exports.isPresentationFailed = isPresentationFailed;
const markProofAsViewed = async (agent, record) => {
  record.metadata.set(_metadata.ProofMetadata.customMetadata, {
    ...record.metadata.data.customMetadata,
    details_seen: true
  });
  return agent.proofs.update(record);
};

/*
 * Add template reference to Proof Exchange record
 * */
exports.markProofAsViewed = markProofAsViewed;
const linkProofWithTemplate = async (agent, record, templateId) => {
  record.metadata.set(_metadata.ProofMetadata.customMetadata, {
    ...record.metadata.data.customMetadata,
    proof_request_template_id: templateId
  });
  return agent.proofs.update(record);
};
exports.linkProofWithTemplate = linkProofWithTemplate;
//# sourceMappingURL=proof.js.map